##################################################
# paired.eseach.R       (Mar 13, 2019)
#   Modified from each.gsea.R (11/7/2014)
#   Bug fix  (3/16/2015)
#   Color enrichment plot (6/4/2015)
#   Combined enrichment score (6/29/2015)
#   Log2ratio output added, CESonly for 2 sets,
#        dir.change, png capability (10/18, 2016)
#   png() to pdf() for GP server (3/13, 2019)
#        comb.marker.name order fixed (4/12,2019)
#
#  inpur: gct, gmx
#    combined score just for gmx with 2 genesets
#    calculated as geneset #2 - geneset #1
#  samples in gct should be ordered like
#    s1-1, s2-1, s3-1,...,s1-2, s2-2, s3-2,..
#    sample names from the first half are used for output
##################################################

paired.eseach <- function(
input.data,  # .gct
marker.data, # .gmx
output.name="peseach",
norm.method="none", # "none" or "row.std"
dir.change="1.over.2", # or "2.over.1"
rank.metric="log2ratio",
gene.ordering="descending", # "ascending" for gct of ranks, "descending" for expression
nperm=1000,
min.size=10,
weighted.score.type=1,
fraction.ES.perm.w.same.sign=0.1,
ces="no", # or "yes" combined enrichment score. only for 2 genesets
plot="T", # or "F"
line.width=2,
line.col="green",
rnd.seed=4736859
)
{

  # for GP
    nperm <- as.numeric(nperm)
    min.size <- as.numeric(min.size)
    weighted.score.type <- as.numeric(weighted.score.type)
    fraction.ES.perm.w.same.sign <- as.numeric(fraction.ES.perm.w.same.sign)
    line.width <- as.numeric(line.width)
    rnd.seed <- as.numeric(rnd.seed)

  set.seed(rnd.seed)

  # Read input data

  d.pair <- read.delim(input.data,header=T,skip=2,check.names=F)
  marker<-read.delim(marker.data,header=T)

  gene.name<-as.vector(d.pair[,1])
  description <- as.vector(d.pair[,2])
  d.pair<-d.pair[-c(1:2)]
  rownames(d.pair)<-gene.name

#print(ranking[1:3,1:3])
  marker.name<-colnames(marker)

  num.sample<-dim(d.pair)[2]/2
    if (dim(d.pair)[2]%%2!=0){
        stop("### Paired gct provided? ###")
    }

  if (dir.change=="1.over.2"){
      sample.name<-paste(colnames(d.pair)[1:num.sample],"_over_",colnames(d.pair)[(num.sample+1):(num.sample*2)],sep="")
  }
  if (dir.change=="2.over.1"){
    sample.name<-paste(colnames(d.pair)[(num.sample+1):(num.sample*2)],"_over_",colnames(d.pair)[1:num.sample],sep="")
  }
  num.gene<-dim(d.pair)[1]
  num.marker<-dim(marker)[2]

  if (ces=="yes" & num.marker!=2){
      stop("### CES is just for 2 gene sets~ ###")
  }

  # normalization
    if (norm.method=="row.std") {
        mean <- apply(d.pair,1,mean,na.rm=T)
        sd <- apply(d.pair,1,sd,na.rm=T)
        d.pair <- (d.pair-mean)/sd
    }

  # take log2ratio
    if (rank.metric=="log2ratio"){
        if (dir.change=="1.over.2"){
            d <- log2(d.pair[,1:num.sample]/d.pair[,(num.sample+1):(num.sample*2)])
        }
        if (dir.change=="2.over.1"){
            d <- log2(d.pair[,(num.sample+1):(num.sample*2)]/d.pair[,1:num.sample])
        }
#        write.table(d,"log2ratio.txt",quote=F,sep="\t",row.names=F)
    }

  # convert to rank
    if (gene.ordering=="descending"){
        ranking <- dim(d)[1]+1-apply(d,2,rank,na.last=F)
    }
    if (gene.ordering=="ascending"){
        ranking <- apply(d,2,rank,na.last=F)
    }
#rownames(ranking)<-gene.name
#print(ranking[1:3,1:3])

  # initialize

  perm.gene <- matrix(0,nrow=num.gene,ncol=nperm)
#print(marker.name)
#print(sample.name)
  if (ces=="no"){
    ES.perm <- vector(length(nperm),mode="numeric")

    ES.obs<-NES <- nom.p <- neg.log10.p.ESsign <-matrix(0,nrow=num.marker,ncol=num.sample)
    rownames(ES.obs)<-rownames(NES) <- rownames(nom.p)<-marker.name
    colnames(ES.obs)<-colnames(NES) <- colnames(nom.p)<-sample.name
  }
  if (ces=="yes" & num.marker==2){
    ES.perm.1 <- ES.perm.2 <- vector(length(nperm),mode="numeric")

    ES.obs <-matrix(0,nrow=2,ncol=num.sample)
    CES.obs <- nom.p <- neg.log10.p.CESsign <-matrix(0,nrow=1,ncol=num.sample)
    comb.marker.name <- paste(marker.name[1],"_minus_",marker.name[2],sep="")
    rownames(CES.obs) <- rownames(nom.p)<- comb.marker.name
    colnames(CES.obs) <- colnames(nom.p)<-sample.name
  }

#print(ES.obs[1:3,1:3])

  # perm gene

  for (i in 1:nperm){
    perm.gene[,i]<-sample(as.vector(gene.name))
  }

  # mapping

if (ces=="no"){

  for (r in 1:num.sample){
print(paste("# Start sample ",r,sep=""))
#    sortlist<-order(ranking[,r],decreasing=T)
    sortlist<-order(ranking[,r],decreasing=F)
    order.gene<-rownames(ranking[sortlist,])
#print(order.gene[1:3])
  if (weighted.score.type==1){
    correl.vector<-as.vector(d[sortlist,r])
  }

    for (m in 1:num.marker){
gene.set<-as.vector(intersect(marker[,m],gene.name))
if (length(gene.set)>=min.size){
#print(gene.set)
      ES.obs[m,r]<-current.ES <- ES(order.gene,gene.set,correl.vector,weighted.score.type,plot=plot,marker.name=marker.name[m],sample.name=sample.name[r],line.width=line.width,line.col=line.col)

      for (p in 1:nperm){
        ES.perm[p]<-ES(perm.gene[,p],gene.set,correl.vector,weighted.score.type)
      }

      # NES
      if (current.ES>0){
          ES.perm.same.sign <- ES.perm[ES.perm>0]
          ES.perm.mean <- mean(ES.perm.same.sign)
      }
      if (current.ES<0){
          ES.perm.same.sign <- ES.perm[ES.perm<0]
          ES.perm.mean <- abs(mean(ES.perm.same.sign))
      }

      if (length(ES.perm.same.sign)>(nperm*fraction.ES.perm.w.same.sign)){
          NES[m,r] <- current.ES/ES.perm.mean
      }else{
          NES[m,r] <- NA
      }

      # nominal p

      if (length(ES.perm.same.sign)>(nperm*fraction.ES.perm.w.same.sign)){
          ES.rank<-rank(c(ES.obs[m,r],ES.perm.same.sign))
          if (ES.obs[m,r]<0){
            nom.p[m,r]<-(ES.rank[1]-1)/(length(ES.rank)-1)
          }else{
            nom.p[m,r]<-(length(ES.rank)-ES.rank[1])/(length(ES.rank)-1)
          }
          if (nom.p[m,r]==0){
            nom.p[m,r]<-1/(length(ES.rank)-1)
          }
      }else{
          ES.rank<-rank(c(ES.obs[m,r],ES.perm))
          if (ES.obs[m,r]<0){
            nom.p[m,r]<-(ES.rank[1]-1)/(length(ES.rank)-1)
          }else{
            nom.p[m,r]<-(length(ES.rank)-ES.rank[1])/(length(ES.rank)-1)
          }
          if (nom.p[m,r]==0){
            nom.p[m,r]<-1/(length(ES.rank)-1)
          }
      }
}else{
ES.obs[m,r]<-NA # gene set < min.size will be NA
NES[m,r] <- NA
nom.p[m,r]<-NA
}
    }
  }   # corresponding to  "for (r in 1:num.sample){"

    # -log10(p) with ES sign
    neg.log10.p.ESsign <- -log10(nom.p)*ES.obs/abs(ES.obs)

}  # correponding to "if (ces=="no){

### ces==yes start
if (ces=="yes"){

  for (r in 1:num.sample){
print(paste("# Start sample ",r,sep=""))
#    sortlist<-order(ranking[,r],decreasing=T)
    sortlist<-order(ranking[,r],decreasing=F)
    order.gene<-rownames(ranking[sortlist,])
#print(order.gene[1:3])
  if (weighted.score.type==1){
    correl.vector<-as.vector(d[sortlist,r])
  }

    for (m in 1:num.marker){
      gene.set<-as.vector(intersect(marker[,m],gene.name))
      if (length(gene.set)<min.size){
        stop("### Gene set size below threshold! ###")
      }
#print(gene.set)
      ES.obs[m,r] <- ES(order.gene,gene.set,correl.vector,weighted.score.type,plot=plot,marker.name=marker.name[m],sample.name=sample.name[r],line.width=line.width,line.col=line.col)

      for (p in 1:nperm){
        if (m==1){
            ES.perm.1[p] <- ES(perm.gene[,p],gene.set,correl.vector,weighted.score.type)
        }
        if (m==2){
            ES.perm.2[p] <- ES(perm.gene[,p],gene.set,correl.vector,weighted.score.type)
        }
      }
    }

#if (dir.change=="1.over.2"){
  CES.obs[1,r] <- ES.obs[1,r]-ES.obs[2,r]
  CES.perm <- ES.perm.1-ES.perm.2
#}
#if (dir.change=="2.over.1"){
#  CES.obs[1,r] <- ES.obs[2,r]-ES.obs[1,r]
#  CES.perm <- ES.perm.2-ES.perm.1
#}
      # nominal p (2-tail)

          CES.rank<-rank(c(CES.obs[1,r],CES.perm))
          if (CES.obs[1,r]>0){
            nom.p[1,r]<-((length(CES.rank)-CES.rank[1])/(length(CES.rank)-1))*2
          }
          if (CES.obs[1,r]<0){
            nom.p[1,r]<-((CES.rank[1]-1)/(length(CES.rank)-1))*2
          }
          if (nom.p[1,r]==0){
            nom.p[1,r]<-1/(length(CES.rank)-1)
          }
          if (nom.p[1,r]>1){
            nom.p[1,r] <- 1
          }

  }   # corresponding to  "for (r in 1:num.sample){"

    # -log10(p) with ES sign
    neg.log10.p.CESsign <- -log10(nom.p)*CES.obs/abs(CES.obs)

}  # correponding to "if (ces=="yes"){

### ces==yes end

#  return(ES.obs,nom.p)

    # Output
if (ces=="no"){
    ## gct of log2ratio, ES, NES, p, -log10(p)wESsign
    colnames(d) <- sample.name
    log2ratio.out <- cbind(gene.name,description,as.matrix(d))
    log2ratio.out <- rbind(colnames(log2ratio.out),log2ratio.out)
    write.table("#1.2",paste(output.name,"_log2ratio.gct",sep=""),quote=F,sep="\t",row.names=F,col.names=F)
    write.table(paste(num.gene,num.sample,sep="\t"),paste(output.name,"_log2ratio.gct",sep=""),quote=F,sep="\t",row.names=F,col.names=F,append=T)
    write.table(log2ratio.out,paste(output.name,"_log2ratio.gct",sep=""),quote=F,sep="\t",row.names=F,col.names=F,append=T)

    ES.obs <- cbind(marker.name,marker.name,ES.obs)
    ES.obs <- rbind(colnames(ES.obs),ES.obs)
    write.table("#1.2",paste(output.name,"_ES.obs.gct",sep=""),quote=F,sep="\t",row.names=F,col.names=F)
    write.table(paste(num.marker,num.sample,sep="\t"),paste(output.name,"_ES.obs.gct",sep=""),quote=F,sep="\t",row.names=F,col.names=F,append=T)
    write.table(ES.obs,paste(output.name,"_ES.obs.gct",sep=""),quote=F,sep="\t",row.names=F,col.names=F,append=T)

    NES <- cbind(marker.name,marker.name,NES)
    NES <- rbind(colnames(NES),NES)
    write.table("#1.2",paste(output.name,"_NES.gct",sep=""),quote=F,sep="\t",row.names=F,col.names=F)
    write.table(paste(num.marker,num.sample,sep="\t"),paste(output.name,"_NES.gct",sep=""),quote=F,sep="\t",row.names=F,col.names=F,append=T)
    write.table(NES,paste(output.name,"_NES.gct",sep=""),quote=F,sep="\t",row.names=F,col.names=F,append=T)

    nom.p <- cbind(marker.name,marker.name,nom.p)
    nom.p <- rbind(colnames(nom.p),nom.p)
    write.table("#1.2",paste(output.name,"_nom.p.gct",sep=""),quote=F,sep="\t",row.names=F,col.names=F)
    write.table(paste(num.marker,num.sample,sep="\t"),paste(output.name,"_nom.p.gct",sep=""),quote=F,sep="\t",row.names=F,col.names=F,append=T)
    write.table(nom.p,paste(output.name,"_nom.p.gct",sep=""),quote=F,sep="\t",row.names=F,col.names=F,append=T)

    neg.log10.p.ESsign <- cbind(marker.name,marker.name,neg.log10.p.ESsign)
    neg.log10.p.ESsign <- rbind(colnames(neg.log10.p.ESsign),neg.log10.p.ESsign)
    write.table("#1.2",paste(output.name,"_neg.log10.p.ESsign.gct",sep=""),quote=F,sep="\t",row.names=F,col.names=F)
    write.table(paste(num.marker,num.sample,sep="\t"),paste(output.name,"_neg.log10.p.ESsign.gct",sep=""),quote=F,sep="\t",row.names=F,col.names=F,append=T)
    write.table(neg.log10.p.ESsign,paste(output.name,"_neg.log10.p.ESsign.gct",sep=""),quote=F,sep="\t",row.names=F,col.names=F,append=T)
}

if (ces=="yes" & num.marker==2){
    colnames(d) <- sample.name
    log2ratio.out <- cbind(gene.name,description,as.matrix(d))
    log2ratio.out <- rbind(colnames(log2ratio.out),log2ratio.out)
    write.table("#1.2",paste(output.name,"_log2ratio.gct",sep=""),quote=F,sep="\t",row.names=F,col.names=F)
    write.table(paste(num.gene,num.sample,sep="\t"),paste(output.name,"_log2ratio.gct",sep=""),quote=F,sep="\t",row.names=F,col.names=F,append=T)
    write.table(log2ratio.out,paste(output.name,"_log2ratio.gct",sep=""),quote=F,sep="\t",row.names=F,col.names=F,append=T)

    CES.obs <- cbind(comb.marker.name,comb.marker.name,CES.obs)
    CES.obs <- rbind(colnames(CES.obs),CES.obs)
    write.table("#1.2",paste(output.name,"_CES.obs.gct",sep=""),quote=F,sep="\t",row.names=F,col.names=F)
    write.table(paste(1,num.sample,sep="\t"),paste(output.name,"_CES.obs.gct",sep=""),quote=F,sep="\t",row.names=F,col.names=F,append=T)
    write.table(CES.obs,paste(output.name,"_CES.obs.gct",sep=""),quote=F,sep="\t",row.names=F,col.names=F,append=T)

    nom.p <- cbind(comb.marker.name,comb.marker.name,nom.p)
    nom.p <- rbind(colnames(nom.p),nom.p)
    write.table("#1.2",paste(output.name,"_nom.p.gct",sep=""),quote=F,sep="\t",row.names=F,col.names=F)
    write.table(paste(1,num.sample,sep="\t"),paste(output.name,"_nom.p.gct",sep=""),quote=F,sep="\t",row.names=F,col.names=F,append=T)
    write.table(nom.p,paste(output.name,"_nom.p.gct",sep=""),quote=F,sep="\t",row.names=F,col.names=F,append=T)

    neg.log10.p.CESsign <- cbind(comb.marker.name,comb.marker.name,neg.log10.p.CESsign)
    neg.log10.p.CESsign <- rbind(colnames(neg.log10.p.CESsign),neg.log10.p.CESsign)
    write.table("#1.2",paste(output.name,"_neg.log10.p.CESsign.gct",sep=""),quote=F,sep="\t",row.names=F,col.names=F)
    write.table(paste(1,num.sample,sep="\t"),paste(output.name,"_neg.log10.p.CESsign.gct",sep=""),quote=F,sep="\t",row.names=F,col.names=F,append=T)
    write.table(neg.log10.p.CESsign,paste(output.name,"_neg.log10.p.CESsign.gct",sep=""),quote=F,sep="\t",row.names=F,col.names=F,append=T)
}

}  # end of Main


# enrichment score

ES <- function(order.rank,order.marker,correl.vector=NULL,weighted.score.type,plot=0,marker.name=NULL,sample.name=NULL,line.width=2,line.col="green")
{

   tag.indicator <- sign(match(order.rank, order.marker, nomatch=0))
   no.tag.indicator <- 1 - tag.indicator
   N <- length(order.rank)
   Nh <- length(order.marker)
   Nm <-  N - Nh

   if (weighted.score.type == 0) {
      correl.vector <- rep(1, N)
   }else{
     alpha <- weighted.score.type
     correl.vector <- abs(correl.vector**alpha)
   }
   sum.correl.tag    <- sum(correl.vector[tag.indicator == 1])
   norm.tag    <- 1.0/sum.correl.tag
   norm.no.tag <- 1.0/Nm
   RES <- cumsum(tag.indicator * correl.vector * norm.tag - no.tag.indicator * norm.no.tag)
   max.ES <- max(RES)
   min.ES <- min(RES)

   # plot

#   if (plot=="T" & capabilities("png")==T){
   if (plot=="T"){
     title<-paste(marker.name,"_on_",sample.name,sep="")
     file.name<-paste(title,".pdf",sep="")
     pdf(file.name)
     par(mar=c(8, 4, 10, 3))
#     par(fin=c(6,4.5))
#       plot(RES,type="l",main=title,lwd=2)
       plot(RES,type="l",lwd=line.width,col=line.col)
       grid()
       abline(h=0,col="gray")
       rug(match(order.marker, order.rank, nomatch=0))
     dev.off()
   }

    rm(RES)
    gc()
   if (max.ES > - min.ES) {
      ES <- max.ES
   } else {
      ES <- min.ES
   }
   return(ES)
}
